/* *****************************************
 *  File name: PWM TMR2　音声再生
 *  8kHz 8bit でサンプルした音声 PCMデータを再生
 *  Notes: 48MHz内部クロック　PLLはOn
 *        PWM出力 RC5
 *        PushSW  RA5
 *
 * Created on May 5, 2020, 6:31 PM
 ***************************************** */

#include <xc.h>
#include "wOha.h"   // 音声PCMデータ 8kHz 8bit

#define _XTAL_FREQ 48000000
#define SW2  RA5

#pragma config FOSC  = INTOSC, WDTE = OFF, PWRTE = OFF, MCLRE = OFF, CP  = OFF
#pragma config BOREN = ON, CLKOUTEN = OFF, IESO  = OFF, FCMEN = OFF
#pragma config WRT = OFF, CPUDIV = NOCLKDIV, USBLSCLK = 48MHz, PLLMULT = 3x
#pragma config PLLEN = ENABLED, STVREN = ON, BORV = LO, LPBOR = OFF, LVP = OFF

void main(void) {
    unsigned int count = 0;
    OSCCON = 0b11111100;     // SPLLEN = ON, 16MHz(x3=48MHz)
    LATC = 0;
    TRISC = 0b11010000;   // PWM, LEDs output pins

    // -------------------------------------------------------
    // Timer2 は、Fosc/4(12MHz)をPreScalerで 1/1
    // 8bitで256カウントする　　　　　12M / 255 = 47KHz
    // PostScaler 1/6 でTMR2IFが「1」になる 7.8KHz周期
    // TMR2IFを常時モニタし、TMR2IFセットで音声データ更新
    // -------------------------------------------------------
   T2CON  = 0b00101100;     // Timer 2 Post 1/6 PS 1/1 設定
   PR2    = 0xFF;           // Timer2 Period Register設定
   PWM1CON= 0b11000000;     // 正論理出力

    while(1){
        count = 0;
        while(wavedata[count]){         // データ終端は「0x00」
            while(! TMR2IF);                // Timer2のカウントアップを待つ
            TMR2IF = 0;                     // フラッグをクリア
            PWM1DCH = wavedata[count++];    // デュティサイクルを設定  
        }
        while (SW2);                    // PushSWが押されるのを待つ
    } 
}
